Attribute VB_Name = "modFileList"
Option Explicit
Private Declare Function GetSystemDirectory Lib _
        "kernel32" Alias "GetSystemDirectoryA" _
        (ByVal lpBuffer As String, _
         ByVal nSize As Long) As Long
Private Declare Function GetWindowsDirectory Lib _
        "kernel32" Alias "GetWindowsDirectoryA" _
        (ByVal lpBuffer As String, _
         ByVal nSize As Long) As Long


'Find files and Dirs APIs
Private Declare Function FindFirstFile Lib "kernel32" Alias _
    "FindFirstFileA" (ByVal lpFileName As String, _
                      lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindNextFile Lib "kernel32" Alias _
    "FindNextFileA" (ByVal hFindFile As Long, _
                     lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindClose Lib "kernel32" (ByVal hFindFile As Long) As Long

Private Const MAX_PATH = 260
Private Const FILE_ATTRIBUTE_ARCHIVE = &H20
Private Const FILE_ATTRIBUTE_COMPRESSED = &H800
Private Const FILE_ATTRIBUTE_DIRECTORY = &H10
Private Const FILE_ATTRIBUTE_HIDDEN = &H2
Private Const FILE_ATTRIBUTE_NORMAL = &H80
Private Const FILE_ATTRIBUTE_READONLY = &H1
Private Const FILE_ATTRIBUTE_SYSTEM = &H4
Private Const FILE_ATTRIBUTE_TEMPORARY = &H100
Private Const FILE_FLAG_BACKUP_SEMANTICS = &H2000000
Private Const INVALID_HANDLE_VALUE = -1&

Private Type FILETIME
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type


Private Type WIN32_FIND_DATA
        dwFileAttributes As Long
        ftCreationTime As FILETIME
        ftLastAccessTime As FILETIME
        ftLastWriteTime As FILETIME
        nFileSizeHigh As Long
        nFileSizeLow As Long
        dwReserved0 As Long
        dwReserved1 As Long
        cFileName As String * MAX_PATH
        cAlternate As String * 14
End Type

Public Type T_FILES
    Path As String
    File As String
End Type

Private Const ARRAY_SIZE = 1000

Private m_lCurIndex As Long

Public Sub ListAllFilesByExt(ByVal sPath As String, _
                              uFiles() As T_FILES, _
                              Optional ByVal sExt As String, _
                              Optional bSearchSubDirs = True)
    '********************************************************
    'Searches for files by given path and file extension.   *
    'Files found are put in the uFiles() array. Optionally  *
    'searches in all subdirs recursively.                   *
    '********************************************************
    'Input:                                                 *
    '       sPath - string, containing the directory to     *
    '               search in.                              *
    '       sExt - string, optional - allows to search for  *
    '              files with given extention.              *
    '       bSearchSubDirs - boolean, optional - if TRUE    *
    '              will cause to search all subdirs under   *
    '              the directory pointed by sPath.          *
    '********************************************************
    'Return:                                                *
    '       uFiles() - array of T_FILES containing the files*
    '                  and their paths.                     *
    '********************************************************

    Dim sFile As String
    Dim hFind As Long
    Dim wfd As WIN32_FIND_DATA
    Dim nExtLen As Integer
    Dim lRetVal As Long
    Dim sTemp As String
    Dim lPos As Long

    If sExt <> "" Then
        If Left$(sExt, 1) <> "." Then
            sExt = "." & sExt
        End If

        nExtLen = Len(sExt)
    End If

    If bSearchSubDirs Then
        If Right$(sPath, 1) <> "\" Then
            sPath = sPath & "\"
        End If

        sFile = sPath & "*.*"

        hFind = FindFirstFile(sFile, wfd)

        If hFind <> 0 Then
            Do
                With wfd
                    lPos = InStr(1, Trim(.cFileName), Chr$(0))
                    If lPos > 0 Then
                        sTemp = Left$(.cFileName, lPos - 1)
                        If (.dwFileAttributes And FILE_ATTRIBUTE_DIRECTORY) = FILE_ATTRIBUTE_DIRECTORY Then
                            'it is another subdir to search
                            If sTemp <> "." And sTemp <> ".." Then
                                'Start searching the new dir recursively
                                ListAllFilesByExt sPath & sTemp, uFiles, sExt
                            End If
                        Else
                            'It is a file
                            If nExtLen > 0 Then
                                'List only files with given extention.
                                If UCase$(Right$(sTemp, nExtLen)) = UCase$(sExt) Then
                                    'increase the array size
                                    ReDim Preserve uFiles(UBound(uFiles) + 1)
                                
                                    'Add the file to the list
                                    uFiles(m_lCurIndex).File = sTemp
                                    uFiles(m_lCurIndex).Path = sPath

                                    'Increment index
                                    m_lCurIndex = m_lCurIndex + 1
                                    
                                    frmExcelReplace.AddToLog ("File Found - " & sPath & sTemp)
                                    
                                    frmExcelReplace.lblFilesFound.Caption = "Files Found : " & m_lCurIndex
                                End If
                            Else
                                'List all files
                                
                                'increase the array size
                                ReDim Preserve uFiles(UBound(uFiles) + 1)

                                'Add the file to the list
                                uFiles(m_lCurIndex).File = sTemp
                                uFiles(m_lCurIndex).Path = sPath

                                'Increment index
                                m_lCurIndex = m_lCurIndex + 1
                                
                                frmExcelReplace.AddToLog ("File Found - " & sPath & sTemp)
                            End If
                        End If  'If (.dwFileAttributes
                    End If  'If lPos > 0
                End With

                'Search for next file
                lRetVal = FindNextFile(hFind, wfd)
                
                DoEvents

            Loop While lRetVal <> 0

            FindClose hFind

        End If  'If hFind <> 0
      Else
        'No Sub Dir search is required.
        If Right$(sPath, 1) <> "\" Then
            sPath = sPath & "\"
        End If

        If nExtLen > 0 Then
            sFile = sPath & "*" & sExt
          Else
            sFile = sPath & "*.*"
        End If

        hFind = FindFirstFile(sFile, wfd)
        If hFind <> 0 Then
            Do
                With wfd
                    lPos = InStr(1, .cFileName, Chr$(0))
                    If lPos > 0 Then
                        sTemp = Left$(.cFileName, lPos - 1)

                        If (.dwFileAttributes And FILE_ATTRIBUTE_DIRECTORY) = 0 Then
                            'It is a file
                            
                            'increase the array size
                            ReDim Preserve uFiles(UBound(uFiles) + 1)
                            
                            'Add the file to the list

                            uFiles(m_lCurIndex).File = sTemp
                            uFiles(m_lCurIndex).Path = sPath

                            'Increment index
                            m_lCurIndex = m_lCurIndex + 1
                            
                            frmExcelReplace.AddToLog ("File Found - " & sPath & sTemp)
                            
                            frmExcelReplace.lblFilesFound.Caption = "Files Found : " & m_lCurIndex
                        End If
                    End If
                End With

                'Search for next file
                lRetVal = FindNextFile(hFind, wfd)
            Loop While lRetVal <> 0

            FindClose hFind
        End If
    End If
End Sub

